package thermo;

/**
 * <p>Title: JThermo</p>
 * <p>Description: </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: www.LancOS.com</p>
 * @author Claudio Lanconelli
 * @version 1.0
 */

import javax.comm.*;
import java.util.Enumeration;

public class SerialConnection  //implements SerialPortEventListener
{
	private CommPortIdentifier portId = null;
	private SerialPort sPort = null;

	private boolean open = false;

	private int portCount = 0;
	private String [] portNameList = null;

	public SerialConnection()
	{
		open = false;

		portNameList = null;
		portCount = 0;
		try {
			for (Enumeration e = CommPortIdentifier.getPortIdentifiers(); e.hasMoreElements(); )
			{
				CommPortIdentifier p = (CommPortIdentifier)e.nextElement();
				if (p.getPortType() == CommPortIdentifier.PORT_SERIAL)
					portCount++;
			}
		}
		catch(NoClassDefFoundError e) {
			e.printStackTrace();
			javax.swing.JOptionPane.showMessageDialog( null,
						"Install Java COMM API or equivalent (RXTX), please",
						"COMM API Classes Not Found",
						javax.swing.JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
		if (portCount > 0)
		{
			portNameList = new String[portCount];
			int i = 0;
			for (Enumeration e = CommPortIdentifier.getPortIdentifiers(); e.hasMoreElements(); )
			{
				CommPortIdentifier p = (CommPortIdentifier)e.nextElement();
				if (p.getPortType() == CommPortIdentifier.PORT_SERIAL)
				{
					portNameList[i] = p.getName();

					//Debug
					System.out.println(portNameList[i]);

					i++;
				}
			}
		}
		else
		{
			javax.swing.JOptionPane.showMessageDialog( null,
						"No COM port found",
						"No COM port found",
						javax.swing.JOptionPane.ERROR_MESSAGE);
			System.exit(1);
		}
	}

	public void openConnection(int portIndex) throws SerialConnectionException
	{
		this.openConnection(portNameList[portIndex]);
	}

	public void openConnection(String portname) throws SerialConnectionException
	{
		// Obtain a CommPortIdentifier object for the port you want to open.
		try {
			portId = CommPortIdentifier.getPortIdentifier(portname);
		} catch (NoSuchPortException e) {
			throw new SerialConnectionException(e.getMessage());
		}

		// Open the port represented by the CommPortIdentifier object. Give
		// the open call a relatively long timeout of 1 seconds to allow
		// a different application to reliquish the port if the user
		// wants to.
		try {
			sPort = (SerialPort)portId.open("JThermo", 100);
		} catch (PortInUseException e) {
	    	throw new SerialConnectionException(e.getMessage());
		}

		// Set the parameters of the connection. If they won't set, close the
		// port before throwing an exception.
		try {
			setConnectionParameters();
		} catch (SerialConnectionException e) {
			sPort.close();
			throw e;
		}
	/**
		// Open the input and output streams for the connection. If they won't
		// open, close the port before throwing an exception.
		try {
			os = sPort.getOutputStream();
			is = sPort.getInputStream();
		} catch (IOException e) {
			sPort.close();
			throw new SerialConnectionException("Error opening i/o streams");
		}

		// Add this object as an event listener for the serial port.
		try {
			sPort.addEventListener(this);
		} catch (TooManyListenersException e) {
			sPort.close();
			throw new SerialConnectionException("too many listeners added");
		}

		// Set notifyOnDataAvailable to true to allow event driven input.
		sPort.notifyOnDataAvailable(true);

		// Set notifyOnBreakInterrup to allow event driven break handling.
		sPort.notifyOnBreakInterrupt(true);

		// Set receive timeout to allow breaking out of polling loop during
		// input handling.
		try {
			sPort.enableReceiveTimeout(30);
		} catch (UnsupportedCommOperationException e) {
		}
	**/

		open = true;
	}

	public void setConnectionParameters() throws SerialConnectionException
	{
		// Save state of parameters before trying a set.
	//	int oldBaudRate = sPort.getBaudRate();
	//	int oldDatabits = sPort.getDataBits();
	//	int oldStopbits = sPort.getStopBits();
	//	int oldParity   = sPort.getParity();
		int oldFlowControl = sPort.getFlowControlMode();
	/**
		// Set connection parameters, if set fails return parameters object
		// to original state.
		try {
			sPort.setSerialPortParams(  parameters.getBaudRate(),
										parameters.getDatabits(),
										parameters.getStopbits(),
										parameters.getParity() );
		} catch (UnsupportedCommOperationException e) {
			parameters.setBaudRate(oldBaudRate);
			parameters.setDatabits(oldDatabits);
			parameters.setStopbits(oldStopbits);
			parameters.setParity(oldParity);
			throw new SerialConnectionException("Unsupported parameter");
		}
	**/
		// Set flow control.
		try {
			sPort.setFlowControlMode(SerialPort.FLOWCONTROL_NONE);
		} catch (UnsupportedCommOperationException e) {
			throw new SerialConnectionException("Unsupported flow control");
		}
	}

	protected void finalize() throws Throwable
	{
		closeConnection();

		super.finalize();
	}

	// Close the port and clean up associated elements.
	public void closeConnection()
	{
		// If port is alread closed just return.
		if (!open)
		{
	    	return;
		}

		// Check to make sure sPort has reference to avoid a NPE.
		if (sPort != null)
		{
		/**
			try {
				// close the i/o streams.
				os.close();
				is.close();
			} catch (IOException e) {
				System.err.println(e);
			}
		**/
			// Close the port.
			sPort.close();
		}

		open = false;
	}

	// Send a one second break signal.
	public void sendBreak(int msec)
	{
		sPort.sendBreak(msec);
	}

	public void setRTS(boolean status)
	{
		sPort.setRTS(status);
	}

	public void setDTR(boolean status)
	{
		sPort.setDTR(status);
	}

	public boolean getCTS()
	{
		return sPort.isCTS();
	}

	public boolean getDSR()
	{
		return sPort.isDSR();
	}

	// Reports the open status of the port.
	public boolean isOpen()
	{
		return open;
	}

	/**
	Handles SerialPortEvents. The two types of SerialPortEvents that this
	program is registered to listen for are DATA_AVAILABLE and BI. During
	DATA_AVAILABLE the port buffer is read until it is drained, when no more
	data is availble and 30ms has passed the method returns. When a BI
	event occurs the words BREAK RECEIVED are written to the messageAreaIn.
	*/

	public void serialEvent(SerialPortEvent e)
	{
		// Create a StringBuffer and int to receive input data.
		StringBuffer inputBuffer = new StringBuffer();
		int newData = 0;

		// Determine type of event.
		switch (e.getEventType())
		{
			// Read data until -1 is returned. If \r is received substitute
			// \n for correct newline handling.
			case SerialPortEvent.DATA_AVAILABLE:
				//newData = is.read();
			break;

			// If break event append BREAK RECEIVED message.
			case SerialPortEvent.BI:
				//"\n--- BREAK RECEIVED ---\n"
		    break;
		}
	}
}